/* This file is part of Disk War, a game by Alessandro Pira */

#include "disk.h"
#include "math.h"
#define ABS(x) (x)>0?(x):-(x)

/***************** CLASS DISCO *****************/
Disco::Disco(int xi, int yi, int acc, int ms)
{
	xs=0;
	ys=0;
	x=xi;
	y=yi;
	accel=acc;
	maxspeed=ms;
	border_x0=9;
	border_x1=630;
	border_y0=9;
	border_y1=470;
};

void Disco::setSourceRect(int x,int y,int l)
{
	source.x=x;
	source.y=y;
	source.h=source.w=l;
	dest.h=dest.w=l;
	radius=l/2;
};

void Disco::SetBorders(int x0,int y0,int x1,int y1)
{
	border_x0=x0;
	border_x1=x1;
	border_y0=y0;
	border_y1=y1;
};

void Disco::LevelSpeed()
{
	if (xs>maxspeed) xs=maxspeed;
	if (xs<-maxspeed) xs= -maxspeed;
	if (ys>maxspeed) ys=maxspeed;
	if (ys<-maxspeed) ys= -maxspeed;
};

void Disco::Move()
{
	x+=(xs / accel);
	y+=(ys / accel);
	if (x<border_x0)
	{
		x=border_x0;
		xs= -xs;
	}
	if (x>border_x1)
	{
		x = border_x1;
		xs= -xs;
	}
	if (y>border_y1)
	{
		y = border_y1;
		ys= -ys;
	}
	if (y<border_y0)
	{
		y=border_y0;
		ys= -ys;
	}
};

SDL_Rect Disco::getDestRect(void)
{
	dest.x=x-radius;
	dest.y=y-radius;
	return dest;
};

void Disco::Clear(SDL_Surface *screen,SDL_Surface *back)
{
	dest.x=x-radius;
	dest.y=y-radius;
	SDL_BlitSurface(back,&dest,screen,&dest);
};

void Disco::Blit(SDL_Surface *screen)
{
	dest.x=x-radius;
	dest.y=y-radius;
	SDL_BlitSurface(bitmap,&source,screen,&dest);
};

/***************** CLASS PLAYER *****************/

Player::Player(int col,int xi,int yi,int maxfire,int maxfuel)
	: Disco(xi,yi)
{
	color=col;
	fire=0;
	fuel=0;
	max_fire=maxfire;
	max_fuel=maxfuel;
	alpha=255;
	status=0;
	action_key[KEY_LEFT]=SDLK_LEFT;
	action_key[KEY_RIGHT]=SDLK_RIGHT;
	action_key[KEY_UP]=SDLK_UP;
	action_key[KEY_DOWN]=SDLK_DOWN;
	action_key[KEY_FIRE]=SDLK_SPACE;
	SetRefuel(-1,-1,-1,-1);
	for (int i=0;i<5;i++)
		keys[i]=0;
	setSourceRect(color*20,0,19);
};

void Player::setKeys(SDLKey up, SDLKey down, SDLKey left, SDLKey right, SDLKey fire)
{
	if (up!=SDLK_NOCHANGE)
		action_key[KEY_UP]=up;
	if (down!=SDLK_NOCHANGE)
		action_key[KEY_DOWN]=down;
	if (left!=SDLK_NOCHANGE)
		action_key[KEY_LEFT]=left;
	if (right!=SDLK_NOCHANGE)
		action_key[KEY_RIGHT]=right;
	if (fire!=SDLK_NOCHANGE)
		action_key[KEY_FIRE]=fire;
};

void Player::ReactToKeys()
{
	if (fuel<max_fuel)
	{
		if (keys[KEY_UP]>0)
		{
			ys--;
			fuel++;
		}
		if (keys[KEY_DOWN]>0)
		{
			ys++;
			fuel++;
		}
		if (keys[KEY_LEFT]>0)
		{
			xs--;
			fuel++;
		}
		if (keys[KEY_RIGHT]>0)
		{
			xs++;
			fuel++;
		}
	}
	if (keys[KEY_FIRE]>0)
	{
		fire++;
	}
	else
	{
		fire-=2;
		if (fire<0) fire=0;
	}
	if (fuel>max_fuel) fuel=max_fuel;
	if (fire>max_fire) fire=max_fire;
	if ( (x>=refuel_x0) && (x<=refuel_x1) && (y>=refuel_y0) && (y<=refuel_y1) )
	{
		fuel-=(max_fuel/100);
		if (fuel<0) fuel=0;
	}
	if ((fuel == max_fuel) && (status == 0))
		status = 3;
	LevelSpeed();
}

// Field of 800x480
// Good for defending
void Player::IA_Move(Disco* bill)
{
	int billx=bill->getX();
	int billy=bill->getY();
	int togox,togoy;
	int tofire;
	int sp_needed;
	int sum;

	togox=x;
	togoy=y;
	tofire=0;

	if (getFuel()<20)
	{
		togox = (refuel_x1 + refuel_x0) / 2;
		togoy = (refuel_y1 + refuel_y0) / 2;
	}
	else if (x<400) // left field
	{
		if (billx>360)
		{
			if (getFuel()>80)
			{
				togoy=billy;
				togox=280;
			}
			else
			{
				togox = (refuel_x1 + refuel_x0) / 2;
				togoy = (refuel_y1 + refuel_y0) / 2;
			}
		}
		else if (billx>80)
		{
			togoy=billy;
			togox=billx+30;
			if ( (billx<110) && (billy>160) && (billy<320) &&
				(bill->getXs()<0) ) tofire=1; // Critical!!
			if ( (bill->getYs()<10) && (x>(billx+5)) ) tofire=1;
		}
		else
		{
			togox=billx+20;
			if (billy>240)
			{
				togoy=billy+15;
				if ( (bill->getYs()<10) && (x>(billx+5)) && (y>billy) ) tofire=1;
			}
			else
			{
				togoy=billy-15;
				if ( (bill->getYs()<10) && (x>(billx+5)) && (y<billy) ) tofire=1;
			}
		}
	}
	else // right field
	{
		if (billx<440)
		{
			if (getFuel()>80)
			{
				togoy=billy;
				togox=520;
			}
			else
			{
				togox = (refuel_x1 + refuel_x0) / 2;
				togoy = (refuel_y1 + refuel_y0) / 2;
			}
		}
		else if (billx<720)
		{
			togoy=billy;
			togox=billx-30;
			if ( (billx>690) && (billy>160) && (billy<320) &&
				(bill->getXs()>0) ) tofire=1; // Critical!!
			if ( (bill->getYs()>-10) && (x<(billx-5)) ) tofire=1;
		}
		else
		{
			togox=billx-20;
			if (billy>240)
			{
				togoy=billy+15;
				if ( (bill->getYs()>-10) && (x<(billx-5)) && (y>billy) ) tofire=1;
			}
			else
			{
				togoy=billy-15;
				if ( (bill->getYs()>-10) && (x<(billx-5)) && (y<billy) ) tofire=1;
			}
		}
	}
	for (int i=0;i<5;i++)
		keys[i]=0;

	sum=0;
	sp_needed=0;
	while ( (sp_needed<maxspeed) && (sum<(ABS(togox-x))) )
	{
		sp_needed++;
		sum+=sp_needed/accel;
	}
	if (x>togox) sp_needed = -sp_needed;
	if (sp_needed<xs) keys[KEY_LEFT]=1;
	if (sp_needed>xs) keys[KEY_RIGHT]=1;

	sum=0;
	sp_needed=0;
	while ( (sp_needed<maxspeed) && (sum<(ABS(togoy-y))) )
	{
		sp_needed++;
		sum+=sp_needed/accel;
	}
	if (y>togoy) sp_needed = -sp_needed;
	if (sp_needed<ys) keys[KEY_UP]=1;
	if (sp_needed>ys) keys[KEY_DOWN]=1;

	if ((tofire>0) && (fire!=max_fire)) keys[KEY_FIRE]=1;

	ReactToKeys();
};

void Player::Key_Move(unsigned char *keymap)
{
	for (int i=0;i<5;i++)
		keys[i]=keymap[action_key[i]];

	ReactToKeys();
};

int Player::FirePower()
{
	if (keys[KEY_FIRE])
		return (max_fire-fire);
	else
		return 0;
};

void Player::Blit(SDL_Surface *screen)
{
	dest.x=x-radius;
	dest.y=y-radius;
	switch (status)
	{
	case 1:
	case 2:
		fire+=5;
		if (fire>max_fire) fire = max_fire;
		alpha--;
		if (alpha < 75)
		{
			alpha--;
			if (alpha <= -100)
			{
				status = 2;
			}
		}
		break;
	case 3:
		fire+=5;
		if (fire>max_fire) fire = max_fire;
		alpha--;
		if (alpha < 75)
		{
			alpha--;
			if (alpha <= -100)
			{
				status = 4;
			}
		}
		break;
	case 4:
		x = (refuel_x0 + refuel_x1) / 2;
		y = (refuel_y0 + refuel_y1) / 2;
		xs = 0;
		ys = 0;
		status = 5;
		break;
	case 5:
		alpha++;
		if (alpha > 180)
		{
			alpha++;
			if (alpha >= 255)
			{
				alpha = 255;
				status = 0;
			}
		}

		break;
	}
	if (alpha >= 0)
	{
		SDL_SetAlpha(bitmap,SDL_SRCALPHA,alpha);
		SDL_BlitSurface(bitmap,&source,screen,&dest);
	}
};

void Player::SetRefuel(int x0, int y0, int x1, int y1)
{
	refuel_x0 = x0;
	refuel_x1 = x1;
	refuel_y0 = y0;
	refuel_y1 = y1;
}

/***************** CLASS BILL *****************/
Bill::Bill(int xi=0,int yi=0)
	: Disco(xi,yi,3)
{
	/* Initialization of a radius 5 disk */
	radius=5;
	setSourceRect(0,0,2*radius);
#ifdef BALL_TRACE
	for (int i=0;i<5;i++)
		lastx[i]=lasty[i]=-1;
#endif
};

void Bill::Attract(Player** pl,int num)
{
	for (int i=0;i<num;i++)
	{
		if (pl[i]->FirePower()>0)
		{
			float dst,acc;
			int tmpx,tmpy;
			tmpx=pl[i]->getX();
			tmpy=pl[i]->getY();
			dst=sqrt((tmpx-x)*(tmpx-x) + (tmpy-y)*(tmpy-y));
			acc=((500-dst)/500)*(pl[i]->FirePower()/20);
			if (acc<0) acc=0;
			if (dst>pl[i]->getRadius())
			{
				xs+=(int)((tmpx-x)*acc/dst);
				ys+=(int)((tmpy-y)*acc/dst);
			}
			else // center is inside player's disk
			{
				xs=pl[i]->getXs();
				ys=pl[i]->getYs();
			}
		}
	}
	LevelSpeed();
};

#ifdef BALL_TRACE
void Bill::Clear(SDL_Surface *screen,SDL_Surface *back)
{
	int i;

	for (i=4;i>=0;i--)
	{
		dest.x=lastx[i]-radius;
		dest.y=lasty[i]-radius;
		SDL_BlitSurface(back,&dest,screen,&dest);
	}

	dest.x=x-radius;
	dest.y=y-radius;
	SDL_BlitSurface(back,&dest,screen,&dest);
}

void Bill::UpdateDest()
{
	lastx[4] = lastx[3];
	lasty[4] = lasty[3];
	lastx[3] = lastx[2];
	lasty[3] = lasty[2];
	lastx[2] = lastx[1];
	lasty[2] = lasty[1];
	lastx[1] = lastx[0];
	lasty[1] = lasty[0];
	lastx[0] = x;
	lasty[0] = y;
}

void Bill::Blit(SDL_Surface *screen)
{
	int i;

	for (i=4;i>=0;i--)
	{
		if ((lastx[i]>=0) && (lasty[i]>=0))
		{
			dest.x=lastx[i]-radius;
			dest.y=lasty[i]-radius;
			switch(i)
			{
			case 4:
				SDL_SetAlpha(bitmap,SDL_SRCALPHA,5);
				break;
			case 3:
				SDL_SetAlpha(bitmap,SDL_SRCALPHA,55);
				break;
			case 2:
				SDL_SetAlpha(bitmap,SDL_SRCALPHA,105);
				break;
			case 1:
				SDL_SetAlpha(bitmap,SDL_SRCALPHA,155);
				break;
			case 0:
				SDL_SetAlpha(bitmap,SDL_SRCALPHA,205);
				break;
			}
			SDL_BlitSurface(bitmap,&source,screen,&dest);
		}
	}

	SDL_SetAlpha(bitmap,SDL_SRCALPHA,255);
	dest.x=x-radius;
	dest.y=y-radius;
	SDL_BlitSurface(bitmap,&source,screen,&dest);
}

void Bill::getDestRect(SDL_Rect ret[6])
{
	int i;

	for (i=0;i<6;i++)
	{
		ret[i].w = dest.w;
		ret[i].h = dest.h;
	}

	ret[0].x=x-radius;
	ret[0].y=y-radius;

	if ((lastx[4]>=0) && (lasty[4]>=0))
	{
		ret[1].x=lastx[4] - radius;
		ret[1].y=lasty[4] - radius;
	}
	else
	{
		ret[1].x=ret[1].y=0;
		ret[1].h=ret[1].w=0;
	}

	if ((lastx[3]>=0) && (lasty[3]>=0))
	{
		ret[2].x=lastx[3] - radius;
		ret[2].y=lasty[3] - radius;
	}
	else
	{
		ret[2].x=ret[2].y=0;
		ret[2].h=ret[2].w=0;
	}

	if ((lastx[2]>=0) && (lasty[2]>=0))
	{
		ret[3].x=lastx[2] - radius;
		ret[3].y=lasty[2] - radius;
	}
	else
	{
		ret[3].x=ret[3].y=0;
		ret[3].h=ret[3].w=0;
	}

	if ((lastx[1]>=0) && (lasty[1]>=0))
	{
		ret[4].x=lastx[1] - radius;
		ret[4].y=lasty[1] - radius;
	}
	else
	{
		ret[4].x=ret[4].y=0;
		ret[4].h=ret[4].w=0;
	}

	if ((lastx[0]>=0) && (lasty[0]>=0))
	{
		ret[5].x=lastx[0] - radius;
		ret[5].y=lasty[0] - radius;
	}
	else
	{
		ret[5].x=ret[5].y=0;
		ret[5].h=ret[5].w=0;
	}
}
#endif

