/* This file is part of Disk War, a game by Alessandro Pira */

#ifndef __DISK_H
#define __DISK_H

#include <SDL/SDL.h>

class Disco {
public:
	Disco(int , int , int=4, int=20);
	void setBitmap(SDL_Surface *bmp) { bitmap=bmp; };
	SDL_Surface *getBitmap(void) { return bitmap; };
	void setSourceRect(int,int,int);
	void setAccel(int acc) { accel=acc; };
	void setMaxSpeed(int ms) { maxspeed=ms; };
	int getRadius(void) { return radius; };
	void setX(int new_x) { x=new_x; };
	int getX(void) { return x; };
	void setY(int new_y) { y=new_y; };
	int getY(void) { return y; };
	void setXs(int s) { xs=s; };
	int getXs(void) { return xs; };
	void setYs(int s) { ys=s; };
	int getYs(void) { return ys; };
	SDL_Rect getDestRect(void);
	void SetBorders(int,int,int,int);
	void LevelSpeed();
	void Move();
	void Clear(SDL_Surface*, SDL_Surface*);
	void Blit(SDL_Surface*);

protected:
	int x,y; /* Central point */
	int xs,ys; /* Speed */
	int radius; /* Needed for blitting (we honly have center coords here */
	int accel;
	int maxspeed;
	SDL_Surface *bitmap;
	SDL_Rect source,dest;
private:
	/* Calculated for the central point */
	int border_x0,border_x1,border_y0,border_y1;
};

/* Array index */
#define KEY_FIRE 0
#define KEY_UP 1
#define KEY_RIGHT 2
#define KEY_DOWN 3
#define KEY_LEFT 4

/* Leave a key unmodified (for function setKeys) */
#define SDLK_NOCHANGE SDLK_LAST

class Player : public Disco {
public:
	Player(int col,int xi=0,int yi=0,int maxfire=100, int maxfuel=1000);
	void setKeys(SDLKey, SDLKey, SDLKey, SDLKey, SDLKey);
	void ReactToKeys();
	void IA_Move(Disco*);
	void Key_Move(unsigned char*);
	int FirePower();
	int getFire() { return ( (max_fire-fire) / (max_fire / 10) ); };
	int getFuel() { return ( (max_fuel-fuel) / (max_fuel / 100) ); };
	int getColor() { return color; };
	void Blit(SDL_Surface*);
	void Blow() { if (status == 0) status = 1; };
	int Blowing() { return (status == 1); };
	void SetRefuel(int,int,int,int);
private:
	int color;
	SDLKey action_key[5];
	int keys[5];
	int max_fire;
	int max_fuel;
	int fuel;
	int fire;
	int alpha; /* alpha blit level */
	int status;
		/* status: 0 - normal
		 *         1 - blowing (alpha must be increased)
		 *         2 - blowing (alpha has reached 256)
		 *         3 - out of fuel (alpha must be increased)
		 *         4 - out of fuel (alpha has reached 256)
		 *         5 - out of fuel (alpha must be decreased) */
	int refuel_x0,refuel_x1,refuel_y0,refuel_y1;
};


class Bill : public Disco {
public:
	Bill(int,int);
	void Attract(Player**,int);
#ifdef BALL_TRACE
	void Clear(SDL_Surface*, SDL_Surface*);
	void Blit(SDL_Surface*);
	void getDestRect(SDL_Rect[6]);
	void UpdateDest();
private:
	int lastx[5];
	int lasty[5];
#endif
};

#endif
