/* This file is part of Disk War, a game by Alessandro Pira */

#include <math.h>
#include <SDL/SDL.h>

#include "game.h"
#include "disk.h"
#include "keys.h"
#include "charmap.h"
#include "init_gfx.h"

/* Directions */
#define DIR_UP 1
#define DIR_RIGHT 2
#define DIR_DOWN 3
#define DIR_LEFT 4

/* Keys */
#define KEY_FIRE 0
#define KEY_UP 1
#define KEY_RIGHT 2
#define KEY_DOWN 3
#define KEY_LEFT 4

/* Acceleration */
#define ACCEL 4

/* Max Speed */
#define MAXSPEED ACCEL*5

#define NUM_PLAYERS 2

SDL_Surface *screen;

int pl_col[NUM_PLAYERS];

int back_map[60]={
0,0,0,0,1,1,0,0,0,0,
0,0,0,0,1,1,0,0,0,0,
-1,0,0,0,1,1,0,0,0,-2,
-1,0,0,0,1,1,0,0,0,-2,
0,0,0,0,1,1,0,0,0,0,
2,0,0,0,1,1,0,0,0,2 };

/* Bars at the bottom of the screen */
int bars[2 * NUM_PLAYERS];
/* bars[0]   -   bars[2]
   bars[1]   -   bars[3] */

/* Score */
int plsco[NUM_PLAYERS];

void init_vars()
{
	pl_col[0] = 0;
	pl_col[1] = 1;

	for (int i=0;i<NUM_PLAYERS;i++)
	{
		plsco[i] = 10;
		bars[2*i] = bars[(2*i)+1] = 0;
	}

	for (int i=0;i<MAP_W*MAP_H;i++)
		if (back_map[i] < 0)
			back_map[i] = pl_col[-(back_map[i]+1)] + 3;
}

void init_disks(Player **pl,Bill **b)
{
	*b = new Bill(MAP_W*40,MAP_H*40);

	for (int i=0;i<NUM_PLAYERS;i++)
	{
		pl[i]=new Player(pl_col[i]);
		pl[i]->setBitmap(tiles_bmp);
	}

	/*********** Player's disks initialization ***********/
	/* Borders */
	pl[0]->SetBorders(9,9,(MAP_W*40)-90,(MAP_H*80)-10);
	pl[1]->SetBorders((MAP_W*40)+90,9,(MAP_W*80)-10,(MAP_H*80)-10);
	/* Refuel */
	pl[0]->SetRefuel(9,(MAP_H*80)-90,90,(MAP_H*80)-10);
	pl[1]->SetRefuel((MAP_W*80)-90,(MAP_H*80)-90,(MAP_W*80)-10,(MAP_H*80)-10);
	/* Keys */
	pl[0]->setKeys(SDLK_NOCHANGE,SDLK_NOCHANGE,SDLK_NOCHANGE,SDLK_NOCHANGE,SDLK_RETURN);
	pl[1]->setKeys(SDLK_w,SDLK_s,SDLK_a,SDLK_d,SDLK_TAB);
	/* Position */
	pl[0]->setX(80);
	pl[0]->setY(MAP_H*40);
	pl[1]->setX(720);
	pl[1]->setY(MAP_H*40);

	/*********** Ball Initialization ***********/
	/* Borders */
	(*b)->setBitmap(bill_bmp);
	(*b)->SetBorders(6,6,(MAP_W*80)-5,(MAP_H*80)-5);
}

int map_pos(int x,int y,int radius)
{
	return ( (x/80)+(y/80)*10 );
}

void make_bar(Player *pl[2])
{
	SDL_Rect src,dest;
	SDL_Rect update[6];

	update[0].h=update[1].h=update[2].h=update[3].h=0;
	update[0].w=update[1].w=update[2].w=update[3].w=0;
	update[0].x=update[1].x=update[2].x=update[3].x=0;
	update[0].y=update[2].y=497;
	update[1].y=update[3].y=557;

	update[4].h = update[5].h = 100;
	update[4].w = update[5].w = 10;
	update[4].y = update[5].y = 490;
	update[4].x = 295;
	update[5].x = 434;

	/* FIRE POWER */

	for (int i=0;i<2;i++)
	{
		while (bars[i*2]>pl[i]->getFire())
		{
			dest.x=50+500*i+20*(bars[i*2]-1);
			dest.y=497;
			dest.h=40;
			dest.w=20;
			if (update[i*2].w==0)
			{
				update[i*2].x=dest.x;
				update[i*2].w=20;
				update[i*2].h=40;
			}
			else
			{
				update[i*2].x-=20;
				update[i*2].w+=20;
			}
			SDL_BlitSurface(back_bmp,&dest,screen,&dest);
			bars[i*2]--;
		}
		while (bars[i*2]<pl[i]->getFire())
		{
			dest.x=50+500*i+20*bars[i*2];
			dest.y=497;
			src.h=dest.h=40;
			src.w=dest.w=20;
			src.y=0;
			src.x=pl[i]->getColor() * 20;
			if (update[i*2].w==0)
			{
				update[i*2].x=dest.x;
				update[i*2].w=20;
				update[i*2].h=40;
			}
			else
			{
				update[i*2].w+=20;
			}
			SDL_BlitSurface(color_tiles_bmp,&src,screen,&dest);
			bars[i*2]++;
		}
	}

	/* POINTS */

	for (int i=0;i<2;i++)
	{
		while (bars[(i*2)+1]>plsco[i])
		{
			dest.x=50+500*i+20*(bars[(i*2)+1]-1);
			dest.y=557;
			dest.h=40;
			dest.w=20;
			if (update[(i*2)+1].x==0)
			{
				update[(i*2)+1].x=dest.x;
				update[(i*2)+1].w=20;
				update[(i*2)+1].h=40;
			}
			else
			{
				update[(i*2)+1].x-=20;
				update[(i*2)+1].w+=20;
			}
			SDL_BlitSurface(back_bmp,&dest,screen,&dest);
			bars[(i*2)+1]--;
		}
		while (bars[(i*2)+1]<plsco[i])
		{
			dest.x=50+500*i+20*(bars[(i*2)+1]);
			dest.y=557;
			src.h=dest.h=40;
			src.w=dest.w=20;
			src.y=0;
			src.x=pl[i]->getColor() * 20;
			if (update[(i*2)+1].x==0)
			{
				update[(i*2)+1].x=dest.x;
				update[(i*2)+1].w=20;
				update[(i*2)+1].h=40;
			}
			else
			{
				update[(i*2)+1].w+=20;
			}
			SDL_BlitSurface(color_tiles_bmp,&src,screen,&dest);
			bars[(i*2)+1]++;
		}
	}

	/* FUEL */

	for (int i=0;i<2;i++)
	{
		dest = update[4+i];
		dest.h = 100 - pl[i]->getFuel();
		SDL_FillRect(screen,&dest,SDL_MapRGB(screen->format,0x0,0x00,0x0));
		dest.y += dest.h;
		dest.h = 100 - dest.h;
		SDL_FillRect(screen,&dest,SDL_MapRGB(screen->format,0x0,0xFF,0x0));
	}

	SDL_UpdateRects(screen,6,update);
}

void ciclo()
{
	int game_over = 0;
	int last_map_pos = -1;
	int tmp;
	int paused = 0;
#ifdef BALL_TRACE
	SDL_Rect bill_update_blit[6];
	SDL_Rect bill_update_clear[6];
#else
	SDL_Rect bill_update[2];
#endif
	Bill *bill;
	SDL_Rect update[2*NUM_PLAYERS];
	Player *disco[NUM_PLAYERS];

	init_disks(disco,&bill);

	update[0].x=update[0].y=0;
	update[0].h=screen->h;
	update[0].w=screen->w;
	SDL_BlitSurface(back_bmp,&update[0],screen,&update[0]);
	SDL_UpdateRects(screen,1,&update[0]);

	while (1)
	{
		for (int i=0;i<NUM_PLAYERS;i++)
			disco[i]->Clear(screen,back_bmp);
		bill->Clear(screen,back_bmp);
		for (int i=0;i<NUM_PLAYERS;i++)
			update[i]=disco[i]->getDestRect();
#ifdef BALL_TRACE
		bill->getDestRect(bill_update_clear);
#else
		bill_update[0]=bill->getDestRect();
#endif

		for (int i=0;i<NUM_PLAYERS;i++)
			disco[i]->Move();
		bill->Move();
#ifdef BALL_TRACE
		bill->UpdateDest();
#endif
		for (int i=0;i<NUM_PLAYERS;i++)
			disco[i]->Blit(screen);
		bill->Blit(screen);
		for (int i=0;i<NUM_PLAYERS;i++)
			update[i+NUM_PLAYERS]=disco[i]->getDestRect();
		SDL_UpdateRects(screen,2*NUM_PLAYERS,update);
#ifdef BALL_TRACE
		bill->getDestRect(bill_update_blit);
		SDL_UpdateRects(screen,6,bill_update_clear);
		SDL_UpdateRects(screen,6,bill_update_blit);
#else
		bill_update[1]=bill->getDestRect();
		SDL_UpdateRects(screen,2,bill_update);
#endif

		tmp=map_pos(bill->getX(),bill->getY(),bill->getRadius());
		if (tmp >= 0)
		{
			if (tmp!=last_map_pos)
			{
				last_map_pos=tmp;
				if (back_map[tmp]>2)
					for (int i=0;i<NUM_PLAYERS;i++)
						if (pl_col[i] == back_map[tmp] - 3)
						{
							plsco[i]--;
							if (plsco[i] < 0) plsco[i] = 0;
						}

				for (int i=0;i<NUM_PLAYERS;i++)
					if (plsco[i] <= 0)
					{
						game_over = 1;
						disco[i]->Blow();
					}

				if (game_over)
				{
					int end=1;

					for (int i=0;i<NUM_PLAYERS;i++)
						if (disco[i]->Blowing())
							end = 0;

					if (end)
					{
						for (int i=0;i<NUM_PLAYERS;i++)
							delete disco[i];
						delete bill;
						return;
					}
				}
			}
		}

		make_bar(disco);

		SDL_Delay(10);

		if (!read_keys())
		{
			for (int i=0;i<NUM_PLAYERS;i++)
				delete disco[i];
			delete bill;
			return;
		}

		if (keymap[SDLK_p] == 1)
		{
			SDL_Rect dst;

			dst.x = 370;
			dst.y = 270;
			dst.h = 15;
			dst.w = 60;
			SDL_FillRect(screen,&dst,SDL_MapRGB(screen->format,0,0,0));
			write_string(screen,"Paused",373,270);
			SDL_UpdateRects(screen,1,&dst);
			paused = 1;
		}

		while (paused)
		{
			if (!read_keys())
			{
				for (int i=0;i<NUM_PLAYERS;i++)
					delete disco[i];
				delete bill;
				return;
			}
			if (keymap[SDLK_p] == 1)
			{
				SDL_Rect dst;

				dst.x = 370;
				dst.y = 270;
				dst.h = 15;
				dst.w = 60;
				SDL_BlitSurface(back_bmp,&dst,screen,&dst);
				SDL_UpdateRects(screen,1,&dst);
				paused = 0;
			}
		}

		for (int i=0;i<NUM_PLAYERS;i++)
		{
			if (i==1)
				disco[i]->IA_Move(bill); //Player vs IA
			else
				disco[i]->Key_Move(keymap);
		}

		bill->Attract(disco,NUM_PLAYERS);
	}
	// Should never reach this
	for (int i=0;i<NUM_PLAYERS;i++)
		delete disco[i];
	delete bill;
}

int main (int argc, char **argv)
{
	init_keys();

	init_vars();

	init_gfx();

	ciclo();

	SDL_Quit();

	return 0;
}

