/* This file is part of Disk War, a game by Alessandro Pira */

#include <SDL/SDL.h>
#include <stdlib.h>

#ifdef USE_XPM

char *findcolor(char *image[],char *color,int colnum,int cpp)
{
	char *ptr=NULL;
	int line,i;
	int ok=0;

	for (line=0;line<colnum && !ok;line++)
	{
		ptr=image[line+1];
		ok=1;
		for (i=0;i<cpp;i++)
		{
			if (*(ptr++)!=color[i]) ok=0;
		}
	}
	if (ok)
	{
		while (*(ptr++) != '#')
			;
		return (ptr);
	}
	else
	{
		return NULL;
	}
}

int gethexvalue(char c)
{
	if ((c>='0') && (c<='9'))
		return (c-'0');
	else if ((c>='a') && (c<='f'))
		return (c-'a'+10);
	else if ((c>='A') && (c<='F'))
		return (c-'A'+10);
	else
		return 0; // Should never happen
}

void setrgb(char *s, unsigned char *r, unsigned char *g, unsigned char *b)
{
	*r =  gethexvalue(*(s++)) * 16;
	*r += gethexvalue(*(s++));

	*g =  gethexvalue(*(s++)) * 16;
	*g += gethexvalue(*(s++));

	*b =  gethexvalue(*(s++)) * 16;
	*b += gethexvalue(*(s++));
}

void setpixel(SDL_Surface *img,int x,int y,char *colorstring)
{
	unsigned char bpp;
	unsigned char *bits;
	unsigned int pixel;
	unsigned char r, g, b;

	setrgb(colorstring,&r,&g,&b);
	pixel=SDL_MapRGB(img->format,r,g,b);

	if (SDL_MUSTLOCK(img))
	{
		if ( SDL_LockSurface(img) < 0 )
		{
			fprintf(stderr, "Can't lock image: %s\n", SDL_GetError());
			exit(1);
		}
	}

	bpp = img->format->BytesPerPixel;
	bits = ((unsigned char *)img->pixels)+y*img->pitch+x*bpp;

	if (SDL_MUSTLOCK(img))
	{
		SDL_UnlockSurface(img);
	}

	switch (bpp)
	{
	case 1:
		*((unsigned char *)(bits)) = (unsigned char)pixel;
		break;
	case 2:
		*((unsigned short *)(bits)) = (unsigned short)pixel;
		break;
	case 3:
		r = (pixel>>img->format->Rshift)&0xFF;
		g = (pixel>>img->format->Gshift)&0xFF;
		b = (pixel>>img->format->Bshift)&0xFF;
		*((bits)+img->format->Rshift/8) = r; 
		*((bits)+img->format->Gshift/8) = g;
		*((bits)+img->format->Bshift/8) = b;
		break;
	case 4:
		*((unsigned int *)(bits)) = (unsigned int)pixel;
		break;
	}
}

SDL_Surface *load_xpm_image(char *xpm[])
{
	SDL_Surface *srf;
	int w,h,cols,cpp;
	int x,y;

	sscanf(xpm[0],"%d %d %d %d",&w,&h,&cols,&cpp);

	srf=SDL_DisplayFormat(SDL_CreateRGBSurface(SDL_SWSURFACE,w,h,16,0xFF,0xFF,0xFF,0));

	for (y=0;y<h;y++)
	{
		int i;
		char *line;
		char *colptr;
		char color[5]; // 5 chars per pixel should be enough for every image

		line=xpm[cols+1+y];
		for (x=0;x<w;x++)
		{
			for (i=0;i<cpp;i++)
			{
				color[i]=*(line++);
			}
			color[cpp]=0;
			colptr=findcolor(xpm,color,cols,cpp);
			setpixel(srf,x,y,colptr);
		}
	}
	return srf;
}

#else

SDL_Surface *load_bmp_image(char *filename)
{
	SDL_Surface *img, *ret;

	img = SDL_LoadBMP(filename);

	if (img == NULL)
	{
		fprintf (stderr,"Can't load %s\n",filename);
		return NULL;
	}

	ret = SDL_DisplayFormat(img);
	SDL_FreeSurface(img);
	return ret;
}

#endif

