/* This file is part of Disk War, a game by Alessandro Pira */

#include <SDL/SDL.h>
#include <stdlib.h>

#include "game.h"
#include "charmap.h"
#include "images.h"

#ifndef BMP_DIR
#define BMP_DIR "bitmaps/"
#endif

#ifdef USE_XPM
#include "xpm/all.h"
#else
char bck_files[8][100]={
        BMP_DIR "backgrounds/back.bmp",     // 0
        BMP_DIR "backgrounds/wall.bmp",     // 1
        BMP_DIR "backgrounds/control.bmp",  // 2
        BMP_DIR "backgrounds/red.bmp",      // 3
        BMP_DIR "backgrounds/green.bmp",    // 4
        BMP_DIR "backgrounds/blue.bmp",     // 5
        BMP_DIR "backgrounds/yellow.bmp",   // 6
        BMP_DIR "backgrounds/purple.bmp" }; // 7
#endif

SDL_Surface *tiles_bmp;
SDL_Surface *color_tiles_bmp;
SDL_Surface *bill_bmp;
SDL_Surface *back_bmp;

void load_gfx()
{
	SDL_Rect src,dest;
	SDL_Surface *bck[8];
	SDL_Surface *img;

#ifdef USE_XPM
	tiles_bmp = load_xpm_image(disks);
	color_tiles_bmp = load_xpm_image(colors);
	bill_bmp = load_xpm_image(bill);
	char_bmp = load_xpm_image(charmap);
	img = load_xpm_image(bar);
	bck[0] = load_xpm_image(back);
	bck[1] = load_xpm_image(wall);
	bck[2] = load_xpm_image(control);
	bck[3] = load_xpm_image(red);
	bck[4] = load_xpm_image(green);
	bck[5] = load_xpm_image(blue);
	bck[6] = load_xpm_image(yellow);
	bck[7] = load_xpm_image(purple);
#else
	tiles_bmp = load_bmp_image(BMP_DIR "disks.bmp");
	color_tiles_bmp = load_bmp_image(BMP_DIR "colors.bmp");
	bill_bmp = load_bmp_image(BMP_DIR "bill.bmp");
	char_bmp = load_bmp_image(BMP_DIR "charmap.bmp");
	img = load_bmp_image(BMP_DIR "bar.bmp");
	for (int i=0;i<8;i++)
		bck[i] = load_bmp_image(bck_files[i]);
#endif

	SDL_SetColorKey(tiles_bmp,SDL_SRCCOLORKEY,SDL_MapRGB(screen->format,0,0,0));
	SDL_SetColorKey(color_tiles_bmp,SDL_SRCCOLORKEY,SDL_MapRGB(screen->format,0xFF,0xFF,0xFF));
	SDL_SetColorKey(bill_bmp,SDL_SRCCOLORKEY,SDL_MapRGB(screen->format,0,0xFF,0));
	SDL_SetColorKey(char_bmp,SDL_SRCCOLORKEY,SDL_MapRGB(screen->format,0,0,0));

	// Create background
	back_bmp = SDL_CreateRGBSurface(SDL_HWSURFACE,screen->w,screen->h,
	                                screen->format->BitsPerPixel,
	                                screen->format->Rmask,
	                                screen->format->Gmask,
	                                screen->format->Bmask,
	                                screen->format->Amask);

	src.x=src.y=0;
	src.h=src.w=80;
	dest.w=dest.h=80;
	for(int x=0;x<MAP_W;x++)
		for(int y=0;y<MAP_H;y++)
		{
			dest.x=x*80;
			dest.y=y*80;
			SDL_BlitSurface(bck[back_map[x+y*MAP_W]],&src,back_bmp,&dest);
		}
	for (int i=0;i<8;i++)
		SDL_FreeSurface (bck[i]);

	src.x=src.y=0;
	src.h=dest.h=img->h;
	src.w=dest.w=img->w;
	dest.x=0;
	dest.y=screen->h-src.h;
	SDL_BlitSurface(img,&src,back_bmp,&dest);
	SDL_FreeSurface (img);

	write_string(back_bmp,"Power",50,482);
	write_string(back_bmp,"Power",550,482);
	write_string(back_bmp,"Life",50,542);
	write_string(back_bmp,"Life",550,542);
	write_string(back_bmp,"Fuel",320,575);
	write_string(back_bmp,"Fuel",459,575);
}

void init_gfx()
{
	if (SDL_Init (SDL_INIT_VIDEO) < 0)
	{
		fprintf (stderr, "couldn't init SDL: %s\n", SDL_GetError ());
		exit (1);
	}
	atexit (SDL_Quit);

	SDL_WM_SetCaption("Disk War","Disk War");
	SDL_ShowCursor(0);

	/* Use hardware surface */
	screen = SDL_SetVideoMode(800,600,16,SDL_HWSURFACE);
	if (screen == NULL)
	{
		fprintf (stderr, "couldn't init Video: %s\n", SDL_GetError ());
		exit (1);
	}

	load_gfx();
}

